Program Beacon;

#pragma CLOCK_FREQ 2457600                        // Change to match xtal frequency

ASM __config _XT_OSC & _WDT_OFF & _PWRTE_OFF & _CP_OFF

// Pin assignment
//   Port A
//     0 : PTT
//     1 : Key1
//     2 : Key2
//     3 : Key3
//     4 : Key4
//
//   Port B                 DIP             DIP On          DIP Off
//     0 : Duty Cycle       4               0 = No pause    1 = Pause
//     1 : Locator          3               0 = JO75IC      1 = JO75KC
//     2 : Send Locator     2               0 = No          1 = Yes
//     3 : Call             1               0 = OZ4UHF/A    1 = OZ4UHF
//     4 : Pause duration   5               0 = 5 s         1 = 1 s
//     5 : QSK (Not impl.)  6               0 = Yes         1 = No
//     6 :                  7
//     7 :                  8 

Const
  PTTPause   =50;                                 // Pause between PTT and Key [ms]
  DitDuration=100;                                // Duration of a dit [ms], 100 ms is equal to 60 LPM
  
Var
  I,J:char;
  
Procedure SendDit(Ch:char);

begin
  Output_port_A(11111b);
  Delay_ms(DitDuration);
  Output_port_A(Ch);
  Delay_ms(DitDuration);
end;

Procedure SendDash(Ch:char);

begin
  Output_port_A(11111b);
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
  Output_port_A(Ch);
  Delay_ms(DitDuration);
end;

Procedure CharacterPause;

begin                                             // Pause is actually 3 x dit but last dit in Send procs.
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
end;

Begin
  Disable_Interrupt('GIE');                       // Initialization of registers
  Clear_Bit('OPTION_REG',7);                      // Enable RB pull up resistors
  Set_Tris_A(00000b);
  Set_Tris_B(11111111b);
  Output_port_A(0);                               // Set all outputs low

  Output_high_port_A(0);                          // PTT On
  Delay_ms(PTTPause);                                    
  
  I:=15;

  While 1 do
  begin
    If I=15 then I:=30; else                      // First bit  11110 = 30
    If I=30 then I:=29; else                      // Second bit 11101 = 29
    If I=29 then I:=27; else                      // Third bit  11011 = 27
    If I=27 then I:=23; else                      // Fourth bit 10111 = 23
    If I=23 then I:=15;                           // Fifth bit  01111 = 15

    If Input_pin_port_B(0)=1 then                 // Pause: 1 = Pause, 0 = No pause
    begin
      Output_port_A(11111b);                      // Key On
      If Input_pin_port_B(4)=0  then
        Delay_s(5);                               // Send carrier
      else
        Delay_s(1);                               // Send carrier
    end;
    Output_port_A(I);                             // Stop sending carrier

    Delay_ms(250);                                // 500 ms pause between keying and ID
    Delay_ms(250);

    SendDash(I);                                  // Send O, ---
    SendDash(I);
    SendDash(I);
    
    CharacterPause;                               // Inter character pause
    
    SendDash(I);                                  // Send Z, --**
    SendDash(I);
    SendDit(I);
    SendDit(I);
    
    CharacterPause;                               // Inter character pause
    
    SendDit(I);                                   // Send 4, ****-
    SendDit(I);
    SendDit(I);
    SendDit(I);
    SendDash;
    
    CharacterPause;                               // Inter character pause
    
    SendDit(I);                                   // Send U, **-
    SendDit(I);
    SendDash;

    CharacterPause;                               // Inter character pause

    SendDit(I);                                   // Send H, ****
    SendDit(I);
    SendDit(I);
    SendDit(I);
        
    CharacterPause;                               // Inter character pause

    SendDit(I);                                   // Send F, **-*
    SendDit(I);
    SendDash(I);
    SendDit(I);
    
    If Input_pin_port_B(3)=0 then                 // If 0 then call = OZ4UHF/A
    begin
      CharacterPause;                             // Inter character pause

      SendDash(I);                                // Send /, -**-*
      SendDit(I);
      SendDit(I);
      SendDash(I);
      SendDit(I);
    
      CharacterPause;                             // Inter character pause
      
      SendDit(I);                                 // Send A, *-
      SendDash(I);
    end;

    If Input_pin_port_B(2)=1 then                 // Send locator if 1
    begin
      CharacterPause;                             // Word pause, 7 x dit
      CharacterPause;                             
      CharacterPause;                             
      
      SendDit(I);                                 // Send J, *---
      SendDash(I);
      SendDash(I);
      SendDash(I);

      CharacterPause;                             // Inter character pause
            
      SendDash(I);                                // Send O, ---
      SendDash(I);
      SendDash(I);
      
      CharacterPause;                             // Inter character pause
      
      SendDash(I);                                // Send 7, --***
      SendDash(I);
      SendDit(I);
      SendDit(I);
      SendDit(I);
              
      CharacterPause;                             // Inter character pause

      SendDit(I);                                 // Send 5, *****
      SendDit(I);
      SendDit(I);
      SendDit(I);
      SendDit(I);
   
      CharacterPause;                             // Inter character pause

      If Input_pin_port_B(1)=1 then               // Send JO75KC if 1, JO75IC if 0
      begin
        SendDash(I);                              // Send K, -*-
        SendDit(I);                                  
        SendDash(I);
    
        CharacterPause;                           // Inter character pause
    
        SendDash(I);                              // Send C, -*-*
        SendDit(I);                                  
        SendDash(I);
        SendDit(I);                                  
      end;
      else
      begin
        SendDit(I);                               // Send I, **
        SendDit(I);                                  
    
        CharacterPause;                           // Inter character pause
    
        SendDash(I);                              // Send C, -*-*
        SendDit(I);                                  
        SendDash(I);
        SendDit(I);                                  
      end;
    end;

    Delay_ms(250);                                // 500 ms pause between ID and keying
    Delay_ms(250);
  end;
End.